@@ -36,50 +36,15 @@
 #include "chanswitch.h"
 #include "channel.h"
 #include "socket.h"
+#include "netinet/in.h"
 #include "xmlrpc-c/abyss.h"
 
+#include "sockutil.h"
+
 #include "socket_unix.h"
 
 
 
-typedef struct {
-    int interruptorFd;
-    int interrupteeFd;
-} interruptPipe;
-
-
-
-static void
-initInterruptPipe(interruptPipe * const pipeP,
-                  const char **   const errorP) {
-
-    int pipeFd[2];
-    int rc;
-
-    rc = pipe(pipeFd);
-
-    if (rc != 0)
-        xmlrpc_asprintf(errorP, "Unable to create a pipe to use to interrupt "
-                        "waits.  pipe() failed with errno %d (%s)",
-                        errno, strerror(errno));
-    else {
-        *errorP = NULL;
-        pipeP->interruptorFd = pipeFd[1];
-        pipeP->interrupteeFd = pipeFd[0];
-    }
-}
-
-
-
-static void
-termInterruptPipe(interruptPipe const pipe) {
-
-    close(pipe.interruptorFd);
-    close(pipe.interrupteeFd);
-}
-
-
-
 struct socketUnix {
 /*----------------------------------------------------------------------------
    The properties/state of a TChanSwitch or TChannel unique to the
@@ -93,39 +58,14 @@ struct socketUnix {
         /* The file descriptor and associated POSIX socket belong to the
            user; we did not create it.
         */
-    interruptPipe interruptPipe;
+    sockutil_InterruptPipe interruptPipe;
+
+    bool isListening;
+        /* We've done a 'listen' on the socket */
 };
 
 
 
-static bool
-connected(int const fd) {
-/*----------------------------------------------------------------------------
-   Return TRUE iff the socket on file descriptor 'fd' is in the connected
-   state.
-
-   If 'fd' does not identify a stream socket or we are unable to determine
-   the state of the stream socket, the answer is "false".
------------------------------------------------------------------------------*/
-    bool connected;
-    struct sockaddr sockaddr;
-    socklen_t nameLen;
-    int rc;
-
-    nameLen = sizeof(sockaddr);
-
-    rc = getpeername(fd, &sockaddr, &nameLen);
-
-    if (rc == 0)
-        connected = TRUE;
-    else
-        connected = FALSE;
-
-    return connected;
-}
-
-
-
 void
 SocketUnixInit(const char ** const errorP) {
 
@@ -150,7 +90,7 @@ channelDestroy(TChannel * const channelP) {
 
     struct socketUnix * const socketUnixP = channelP->implP;
 
-    termInterruptPipe(socketUnixP->interruptPipe);
+    sockutil_interruptPipeTerm(socketUnixP->interruptPipe);
 
     if (!socketUnixP->userSuppliedFd)
         close(socketUnixP->fd);
@@ -175,12 +115,18 @@ channelWrite(TChannel *            const channelP,
 
     assert(sizeof(size_t) >= sizeof(len));
 
-    for (bytesLeft = len, error = FALSE;
+    for (bytesLeft = len, error = false;
          bytesLeft > 0 && !error;
         ) {
         size_t const maxSend = (size_t)(-1) >> 1;
 
         ssize_t rc;
+
+        // We'd like to use MSG_NOSIGNAL here, to prevent this send() from
+        // causing a SIGPIPE if the other end of the socket is closed, but
+        // MSG_NOSIGNAL is not standard enough.  An SO_NOSIGPIPE socket
+        // option is another way, but even less standard.  So instead, the
+        // thread simply must be set to ignore SIGPIPE.
         
         rc = send(socketUnixP->fd, &buffer[len-bytesLeft],
                   MIN(maxSend, bytesLeft), 0);
@@ -202,7 +148,7 @@ channelWrite(TChannel *            const channelP,
         }
         if (rc <= 0)
             /* 0 means connection closed; < 0 means severe error */
-            error = TRUE;
+            error = true;
         else
             bytesLeft -= rc;
     }
@@ -226,14 +172,14 @@ channelRead(TChannel *      const channelP,
     rc = recv(socketUnixP->fd, buffer, bufferSize, 0);
 
     if (rc < 0) {
-        *failedP = TRUE;
+        *failedP = true;
         if (ChannelTraceIsActive)
             fprintf(stderr, "Abyss channel: "
                     "Failed to receive data from socket.  "
                     "recv() failed with errno %d (%s)\n",
                     errno, strerror(errno));
     } else {
-        *failedP = FALSE;
+        *failedP = false;
         *bytesReceivedP = rc;
 
         if (ChannelTraceIsActive)
@@ -274,12 +220,13 @@ channelWait(TChannel * const channelP,
    for most purposes).
 
    Return *readyToReadP == true if the reason for returning is that
-   the channel is immediately readable.  *readyToWriteP is analogous
+   the channel is immediately readable.  But only if 'readyToReadP' is
+   non-null.  *readyToWriteP is analogous
    for writable.  Both may be true.
 
-   Return *failedP true iff we fail to wait for the requested
-   condition because of some unusual problem.  Being interrupted by a
-   signal is not a failure.
+   Return *failedP true iff we fail to wait for the requested condition
+   because of some unusual problem and 'failedP' is non-null.  Being
+   interrupted by a signal is not a failure.
 
    If one of these return value pointers is NULL, don't return that
    value.
@@ -312,16 +259,16 @@ channelWait(TChannel * const channelP,
 
     if (rc < 0) {
         if (errno == EINTR) {
-            failed       = FALSE;
-            readyToRead  = FALSE;
-            readyToWrite = FALSE;
+            failed       = false;
+            readyToRead  = false;
+            readyToWrite = false;
         } else {
-            failed       = TRUE;
-            readyToRead  = FALSE; /* quiet compiler warning */
-            readyToWrite = FALSE; /* quiet compiler warning */
+            failed       = true;
+            readyToRead  = false; /* quiet compiler warning */
+            readyToWrite = false; /* quiet compiler warning */
         }
     } else {
-        failed       = FALSE;
+        failed       = false;
         readyToRead  = !!(pollfds[0].revents & POLLIN);
         readyToWrite = !!(pollfds[0].revents & POLLOUT);
     }
@@ -348,9 +295,8 @@ channelInterrupt(TChannel * const channelP) {
   again wait.
 -----------------------------------------------------------------------------*/
     struct socketUnix * const socketUnixP = channelP->implP;
-    unsigned char const zero[1] = {0u};
 
-    write(socketUnixP->interruptPipe.interruptorFd, &zero, sizeof(zero));
+    sockutil_interruptPipeInterrupt(socketUnixP->interruptPipe);
 }
 
 
@@ -363,41 +309,7 @@ ChannelUnixGetPeerName(TChannel *         const channelP,
 
     struct socketUnix * const socketUnixP = channelP->implP;
 
-    unsigned char * peerName;
-    socklen_t nameSize;
-
-    nameSize = sizeof(struct sockaddr) + 1;
-    
-    peerName = malloc(nameSize);
-
-    if (peerName == NULL)
-        xmlrpc_asprintf(errorP, "Unable to allocate space for peer name");
-    else {
-        int rc;
-        socklen_t nameLen;
-        nameLen = nameSize;  /* initial value */
-        rc = getpeername(socketUnixP->fd, (struct sockaddr *)peerName,
-                         &nameLen);
-
-        if (rc < 0)
-            xmlrpc_asprintf(errorP, "getpeername() failed.  errno=%d (%s)",
-                            errno, strerror(errno));
-        else {
-            if (nameLen > nameSize-1)
-                xmlrpc_asprintf(errorP,
-                                "getpeername() says the socket name is "
-                                "larger than %u bytes, which means it is "
-                                "not in the expected format.",
-                                nameSize-1);
-            else {
-                *sockaddrPP = (struct sockaddr *)peerName;
-                *sockaddrLenP = nameLen;
-                *errorP = NULL;
-            }
-        }
-        if (*errorP)
-            free(peerName);
-    }
+    sockutil_getPeerName(socketUnixP->fd, sockaddrPP, sockaddrLenP, errorP);
 }
 
 
@@ -410,37 +322,7 @@ channelFormatPeerInfo(TChannel *    const channelP,
 
     struct socketUnix * const socketUnixP = channelP->implP;
 
-    struct sockaddr sockaddr;
-    socklen_t sockaddrLen;
-    int rc;
-
-    sockaddrLen = sizeof(sockaddr);
-    
-    rc = getpeername(socketUnixP->fd, &sockaddr, &sockaddrLen);
-    
-    if (rc < 0)
-        xmlrpc_asprintf(peerStringP, "?? getpeername() failed.  errno=%d (%s)",
-                        errno, strerror(errno));
-    else {
-        switch (sockaddr.sa_family) {
-        case AF_INET: {
-            struct sockaddr_in * const sockaddrInP =
-                (struct sockaddr_in *) &sockaddr;
-            if (sockaddrLen < sizeof(*sockaddrInP))
-                xmlrpc_asprintf(peerStringP, "??? getpeername() returned "
-                                "the wrong size");
-            else {
-                unsigned char * const ipaddr = (unsigned char *)
-                    &sockaddrInP->sin_addr.s_addr;
-                xmlrpc_asprintf(peerStringP, "%u.%u.%u.%u:%hu",
-                                ipaddr[0], ipaddr[1], ipaddr[2], ipaddr[3],
-                                sockaddrInP->sin_port);
-            }
-        } break;
-        default:
-            xmlrpc_asprintf(peerStringP, "??? AF=%u", sockaddr.sa_family);
-        }
-    }
+    sockutil_formatPeerInfo(socketUnixP->fd, peerStringP);
 }
 
 
@@ -495,9 +377,9 @@ makeChannelFromFd(int           const fd,
         TChannel * channelP;
         
         socketUnixP->fd = fd;
-        socketUnixP->userSuppliedFd = TRUE;
+        socketUnixP->userSuppliedFd = true;
 
-        initInterruptPipe(&socketUnixP->interruptPipe, errorP);
+        sockutil_interruptPipeInit(&socketUnixP->interruptPipe, errorP);
 
         if (!*errorP) {
             ChannelCreate(&channelVtbl, socketUnixP, &channelP);
@@ -510,7 +392,7 @@ makeChannelFromFd(int           const fd,
                 *errorP = NULL;
             }
             if (*errorP)
-                termInterruptPipe(socketUnixP->interruptPipe);
+                sockutil_interruptPipeTerm(socketUnixP->interruptPipe);
         }
         if (*errorP)
             free(socketUnixP);
@@ -525,28 +407,29 @@ ChannelUnixCreateFd(int                           const fd,
                     struct abyss_unix_chaninfo ** const channelInfoPP,
                     const char **                 const errorP) {
 
-    struct sockaddr peerAddr;
-    socklen_t peerAddrLen;
-    int rc;
+    if (!sockutil_connected(fd))
+        xmlrpc_asprintf(errorP, "Socket on file descriptor %d is not in "
+                        "connected state.", fd);
+    else {
+        struct sockaddr * peerAddrP;
+        size_t peerAddrLen;
+        const char * error;
 
-    peerAddrLen = sizeof(peerAddr);
+        sockutil_getPeerName(fd, &peerAddrP, &peerAddrLen, &error);
 
-    rc = getpeername(fd, &peerAddr, &peerAddrLen);
+        if (error) {
+            xmlrpc_asprintf(errorP, "Failed to get identity of client.  %s",
+                            error);
+            xmlrpc_strfree(error);
+        } else {
+            makeChannelInfo(channelInfoPP, *peerAddrP, peerAddrLen, errorP);
+            if (!*errorP) {
+                makeChannelFromFd(fd, channelPP, errorP);
 
-    if (rc != 0) {
-        if (errno == ENOTCONN)
-            xmlrpc_asprintf(errorP, "Socket on file descriptor %d is not in "
-                            "connected state.", fd);
-        else
-            xmlrpc_asprintf(errorP, "getpeername() failed on fd %d.  "
-                            "errno=%d (%s)", fd, errno, strerror(errno));
-    } else {
-        makeChannelInfo(channelInfoPP, peerAddr, peerAddrLen, errorP);
-        if (!*errorP) {
-            makeChannelFromFd(fd, channelPP, errorP);
-
-            if (*errorP)
-                free(*channelInfoPP);
+                if (*errorP)
+                    free(*channelInfoPP);
+            }
+            free(peerAddrP);
         }
     }
 }
@@ -564,7 +447,7 @@ chanSwitchDestroy(TChanSwitch * const chanSwitchP) {
 
     struct socketUnix * const socketUnixP = chanSwitchP->implP;
 
-    termInterruptPipe(socketUnixP->interruptPipe);
+    sockutil_interruptPipeTerm(socketUnixP->interruptPipe);
 
     if (!socketUnixP->userSuppliedFd)
         close(socketUnixP->fd);
@@ -574,7 +457,7 @@ chanSwitchDestroy(TChanSwitch * const chanSwitchP) {
 
 
 
-static SwitchListenImpl  chanSwitchListen;
+static SwitchListenImpl chanSwitchListen;
 
 static void
 chanSwitchListen(TChanSwitch * const chanSwitchP,
@@ -583,68 +466,13 @@ chanSwitchListen(TChanSwitch * const chanSwitchP,
 
     struct socketUnix * const socketUnixP = chanSwitchP->implP;
 
-    int32_t const minus1 = -1;
+    if (socketUnixP->isListening)
+        xmlrpc_asprintf(errorP, "Channel switch is already listening");
+    else {
+        sockutil_listen(socketUnixP->fd, backlog, errorP);
 
-    int rc;
-
-    /* Disable the Nagle algorithm to make persistant connections faster */
-
-    setsockopt(socketUnixP->fd, IPPROTO_TCP, TCP_NODELAY,
-               &minus1, sizeof(minus1));
-
-    rc = listen(socketUnixP->fd, backlog);
-
-    if (rc == -1)
-        xmlrpc_asprintf(errorP, "listen() failed with errno %d (%s)",
-                        errno, strerror(errno));
-    else
-        *errorP = NULL;
-}
-
-
-
-static void
-waitForConnection(struct socketUnix * const listenSocketP,
-                  bool *              const interruptedP,
-                  const char **       const errorP) {
-/*----------------------------------------------------------------------------
-   Wait for the listening socket to have a connection ready to accept.
-
-   We return before the requested condition holds if the process receives
-   (and catches) a signal, but only if it receives that signal a certain
-   time after we start running.  (That means this behavior isn't useful
-   for most purposes).
-
-   We furthermore return before the requested condition holds if someone sends
-   a byte through the listening socket's interrupt pipe (or has sent one
-   previously since the most recent time the pipe was drained).
-
-   Return *interruptedP == true if we return before there is a connection
-   ready to accept.
------------------------------------------------------------------------------*/
-    struct pollfd pollfds[2];
-    int rc;
-
-    pollfds[0].fd = listenSocketP->fd;
-    pollfds[0].events = POLLIN;
-
-    pollfds[1].fd = listenSocketP->interruptPipe.interrupteeFd;
-    pollfds[1].events = POLLIN;
-    
-    rc = poll(pollfds, ARRAY_SIZE(pollfds), -1);
-
-    if (rc < 0) {
-        if (errno == EINTR) {
-            *errorP       = NULL;
-            *interruptedP = TRUE;
-        } else {
-            xmlrpc_asprintf(errorP, "poll() failed, errno = %d (%s)",
-                            errno, strerror(errno));
-            *interruptedP = FALSE; /* quiet compiler warning */
-        }
-    } else {
-        *errorP       = NULL;
-        *interruptedP = !(pollfds[0].revents & POLLIN);
+        if (!*errorP)
+            socketUnixP->isListening = true;
     }
 }
 
@@ -676,9 +504,10 @@ createChannelForAccept(int             const acceptedFd,
             xmlrpc_asprintf(errorP, "Unable to allocate memory");
         else {
             acceptedSocketP->fd = acceptedFd;
-            acceptedSocketP->userSuppliedFd = FALSE;
+            acceptedSocketP->userSuppliedFd = false;
 
-            initInterruptPipe(&acceptedSocketP->interruptPipe, errorP);
+            sockutil_interruptPipeInit(
+                &acceptedSocketP->interruptPipe, errorP);
 
             if (!*errorP) {
                 TChannel * channelP;
@@ -693,7 +522,7 @@ createChannelForAccept(int             const acceptedFd,
                     *channelInfoPP = channelInfoP;
                 }
                 if (*errorP)
-                    termInterruptPipe(acceptedSocketP->interruptPipe);
+                    sockutil_interruptPipeTerm(acceptedSocketP->interruptPipe);
             }
             if (*errorP)
                 free(acceptedSocketP);
@@ -726,20 +555,24 @@ chanSwitchAccept(TChanSwitch * const chanSwitchP,
     bool interrupted;
     TChannel * channelP;
 
-    interrupted = FALSE; /* Haven't been interrupted yet */
+    interrupted = false; /* Haven't been interrupted yet */
     channelP    = NULL;  /* No connection yet */
     *errorP     = NULL;  /* No error yet */
 
     while (!channelP && !*errorP && !interrupted) {
 
-        waitForConnection(listenSocketP, &interrupted, errorP);
+        sockutil_waitForConnection(listenSocketP->fd,
+                                   listenSocketP->interruptPipe,
+                                   &interrupted, errorP);
 
         if (!*errorP && !interrupted) {
             struct sockaddr peerAddr;
-            socklen_t size = sizeof(peerAddr);
+            socklen_t peerAddrLen;
             int rc;
 
-            rc = accept(listenSocketP->fd, &peerAddr, &size);
+            peerAddrLen = sizeof(peerAddr);  /* initial value */
+
+            rc = accept(listenSocketP->fd, &peerAddr, &peerAddrLen);
 
             if (rc >= 0) {
                 int const acceptedFd = rc;
@@ -750,7 +583,7 @@ chanSwitchAccept(TChanSwitch * const chanSwitchP,
                 if (*errorP)
                     close(acceptedFd);
             } else if (errno == EINTR)
-                interrupted = TRUE;
+                interrupted = true;
             else
                 xmlrpc_asprintf(errorP, "accept() failed, errno = %d (%s)",
                                 errno, strerror(errno));
@@ -795,10 +628,21 @@ createChanSwitch(int            const fd,
                  bool           const userSuppliedFd,
                  TChanSwitch ** const chanSwitchPP,
                  const char **  const errorP) {
+/*----------------------------------------------------------------------------
+   Create a channel switch from the bound, but not yet listening, socket
+   with file descriptor 'fd'.
 
+   Return the handle of the new channel switch as *chanSwitchPP.
+
+   'userSuppliedFd' means the file descriptor (socket) shall _not_ belong to
+   the channel switch, so destroying the channel switch does not close it.
+-----------------------------------------------------------------------------*/
     struct socketUnix * socketUnixP;
 
-    assert(!connected(fd));
+    assert(!sockutil_connected(fd));
+
+    if (SwitchTraceIsActive)
+        fprintf(stderr, "Creating Unix listen-socket based channel switch\n");
 
     MALLOCVAR(socketUnixP);
 
@@ -810,13 +654,14 @@ createChanSwitch(int            const fd,
 
         socketUnixP->fd = fd;
         socketUnixP->userSuppliedFd = userSuppliedFd;
+        socketUnixP->isListening = false;
             
-        initInterruptPipe(&socketUnixP->interruptPipe, errorP);
+        sockutil_interruptPipeInit(&socketUnixP->interruptPipe, errorP);
 
         if (!*errorP) {
             ChanSwitchCreate(&chanSwitchVtbl, socketUnixP, &chanSwitchP);
             if (*errorP)
-                termInterruptPipe(socketUnixP->interruptPipe);
+                sockutil_interruptPipeTerm(socketUnixP->interruptPipe);
 
             if (chanSwitchP == NULL)
                 xmlrpc_asprintf(errorP, "Unable to allocate memory for "
@@ -834,76 +679,58 @@ createChanSwitch(int            const fd,
 
 
 static void
-setSocketOptions(int           const fd,
-                 const char ** const errorP) {
-
-    int32_t n = 1;
-    int rc;
-
-    rc = setsockopt(fd, SOL_SOCKET, SO_REUSEADDR, (char*)&n, sizeof(n));
-
-    if (rc < 0)
-        xmlrpc_asprintf(errorP, "Failed to set socket options.  "
-                        "setsockopt() failed with errno %d (%s)",
-                        errno, strerror(errno));
-    else
-        *errorP = NULL;
-}
-
-
-
-static void
-bindSocketToPort(int              const fd,
-                 struct in_addr * const addrP,
-                 uint16_t         const portNumber,
-                 const char **    const errorP) {
-
-    struct sockaddr_in name;
-    int rc;
-
-    name.sin_family = AF_INET;
-    name.sin_port   = htons(portNumber);
-    if (addrP)
-        name.sin_addr = *addrP;
-    else
-        name.sin_addr.s_addr = INADDR_ANY;
-
-    rc = bind(fd, (struct sockaddr *)&name, sizeof(name));
-
-    if (rc == -1)
-        xmlrpc_asprintf(errorP, "Unable to bind socket to port number %hu.  "
-                        "bind() failed with errno %d (%s)",
-                        portNumber, errno, strerror(errno));
-    else
-        *errorP = NULL;
-}
-
-
-
-void
-ChanSwitchUnixCreate(unsigned short const portNumber,
-                     TChanSwitch ** const chanSwitchPP,
-                     const char **  const errorP) {
+switchCreateIpV4Port(unsigned short          const portNumber,
+                     TChanSwitch **          const chanSwitchPP,
+                     const char **           const errorP) {
 /*----------------------------------------------------------------------------
-   Create a POSIX-socket-based channel switch.
+   Create a POSIX-socket-based channel switch for an IPv4 endpoint.
 
-   Use an IP socket.
-
-   Set the socket's local address so that a subsequent "listen" will listen
-   on all IP addresses, port number 'portNumber'.
+   Set the socket's local address so that a subsequent "listen" will listen on
+   all interfaces, port number 'portNumber'.
 -----------------------------------------------------------------------------*/
     int rc;
-    rc = socket(AF_INET, SOCK_STREAM, 0);
+    rc = socket(PF_INET, SOCK_STREAM, 0);
     if (rc < 0)
         xmlrpc_asprintf(errorP, "socket() failed with errno %d (%s)",
                         errno, strerror(errno));
     else {
         int const socketFd = rc;
 
-        setSocketOptions(socketFd, errorP);
+        sockutil_setSocketOptions(socketFd, errorP);
         if (!*errorP) {
-            bindSocketToPort(socketFd, NULL, portNumber, errorP);
-                
+            sockutil_bindSocketToPortInet(socketFd, portNumber, errorP);
+
+            if (!*errorP) {
+                bool const userSupplied = false;
+                createChanSwitch(socketFd, userSupplied, chanSwitchPP, errorP);
+            }
+        }
+        if (*errorP)
+            close(socketFd);
+    }
+}
+
+
+
+static void
+switchCreateIpV6Port(unsigned short          const portNumber,
+                     TChanSwitch **          const chanSwitchPP,
+                     const char **           const errorP) {
+/*----------------------------------------------------------------------------
+  Same as switchCreateIpV4Port(), except for IPv6.
+-----------------------------------------------------------------------------*/
+    int rc;
+    rc = socket(PF_INET6, SOCK_STREAM, 0);
+    if (rc < 0)
+        xmlrpc_asprintf(errorP, "socket() failed with errno %d (%s)",
+                        errno, strerror(errno));
+    else {
+        int const socketFd = rc;
+
+        sockutil_setSocketOptions(socketFd, errorP);
+        if (!*errorP) {
+            sockutil_bindSocketToPortInet6(socketFd, portNumber, errorP);
+
             if (!*errorP) {
                 bool const userSupplied = false;
                 createChanSwitch(socketFd, userSupplied, chanSwitchPP, errorP);
@@ -916,12 +743,69 @@ ChanSwitchUnixCreate(unsigned short const portNumber,
 
 
 
+void
+ChanSwitchUnixCreate(unsigned short const portNumber,
+                     TChanSwitch ** const chanSwitchPP,
+                     const char **  const errorP) {
+
+    switchCreateIpV4Port(portNumber, chanSwitchPP, errorP);
+}
+
+
+
+void
+ChanSwitchUnixCreate2(int                     const protocolFamily,
+                      const struct sockaddr * const sockAddrP,
+                      socklen_t               const sockAddrLen,
+                      TChanSwitch **          const chanSwitchPP,
+                      const char **           const errorP) {
+
+    int rc;
+    rc = socket(protocolFamily, SOCK_STREAM, 0);
+    if (rc < 0)
+        xmlrpc_asprintf(errorP, "socket() failed with errno %d (%s)",
+                        errno, strerror(errno));
+    else {
+        int const socketFd = rc;
+
+        if (SwitchTraceIsActive)
+            fprintf(stderr, "Created socket for protocol family %d\n",
+                    protocolFamily);
+
+        sockutil_setSocketOptions(socketFd, errorP);
+        if (!*errorP) {
+            sockutil_bindSocketToPort(socketFd, sockAddrP, sockAddrLen,
+                                      errorP);
+
+            if (!*errorP) {
+                bool const userSupplied = false;
+                createChanSwitch(socketFd, userSupplied, chanSwitchPP, errorP);
+            }
+        }
+        if (*errorP)
+            close(socketFd);
+    }
+
+}
+
+
+
+void
+ChanSwitchUnixCreateIpV6Port(unsigned short const portNumber,
+                             TChanSwitch ** const chanSwitchPP,
+                             const char **  const errorP) {
+
+    switchCreateIpV6Port(portNumber, chanSwitchPP, errorP);
+}
+
+
+
 void
 ChanSwitchUnixCreateFd(int            const fd,
                        TChanSwitch ** const chanSwitchPP,
                        const char **  const errorP) {
 
-    if (connected(fd))
+    if (sockutil_connected(fd))
         xmlrpc_asprintf(errorP,
                         "Socket (file descriptor %d) is in connected "
                         "state.", fd);
@@ -933,6 +817,27 @@ ChanSwitchUnixCreateFd(int            const fd,
 
 
 
+void
+ChanSwitchUnixGetListenName(TChanSwitch *      const chanSwitchP,
+                            struct sockaddr ** const sockaddrPP,
+                            size_t  *          const sockaddrLenP,
+                            const char **      const errorP) {
+/*----------------------------------------------------------------------------
+   The primary case where this is useful is where the user created the channel
+   switch with a parameters telling the OS to pick the TCP port.  In that
+   case, this is the only way the user can find out what port the OS picked.
+-----------------------------------------------------------------------------*/
+    struct socketUnix * const socketUnixP = chanSwitchP->implP;
+    
+    if (!socketUnixP->isListening)
+        xmlrpc_asprintf(errorP, "Channel Switch is not listening");
+    else
+        sockutil_getSockName(socketUnixP->fd,
+                             sockaddrPP, sockaddrLenP, errorP);
+}
+
+
+
 /*=============================================================================
       obsolete TSocket interface
 =============================================================================*/
@@ -945,7 +850,7 @@ SocketUnixCreateFd(int        const fd,
     TSocket * socketP;
     const char * error;
 
-    if (connected(fd)) {
+    if (sockutil_connected(fd)) {
         TChannel * channelP;
         struct abyss_unix_chaninfo * channelInfoP;
         ChannelUnixCreateFd(fd, &channelP, &channelInfoP, &error);
@@ -963,3 +868,6 @@ SocketUnixCreateFd(int        const fd,
     } else
         *socketPP = socketP;
 }
+
+
+
